<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Controller;

/**
 * @package     imia_base_ext
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class ActionController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * @var array
     */
    protected $fileStorage = array();

    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    protected function translate($key, $arguments = array())
    {
        return (string)\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate($key, $this->extensionName, $arguments);
    }

    /**
     * Load arguments for a different plugin than the current
     *
     * @param string $pluginName
     */
    protected function loadArguments($pluginName = null)
    {
        if ($pluginName) {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::_GPmerged($this->getPluginNamespace($pluginName));
        } else {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::array_merge_recursive_overrule($_GET, $_POST);
        }

        $this->request->setArguments($arguments);
    }

    /**
     *
     * @param string $pluginName
     * @param string $extensionName
     * @return string
     */
    protected function getPluginNamespace($pluginName = null, $extensionName = null)
    {
        $extensionService = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Service\\ExtensionService');
        $pluginNamespace = $extensionService->getPluginNamespace(
                $extensionName ? $extensionName : $this->extensionName,
                $pluginName ? $pluginName : $this->request->getPluginName()
        );

        return $pluginNamespace;
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $body
     * @param string $format
     * @return boolean
     */
    protected function sendMail(array $recipient, array $sender, $subject, $body, $format = 'text')
    {
        $message = $this->objectManager->get('TYPO3\CMS\Core\Mail\MailMessage');

        $message->setTo($recipient)
            ->setFrom($sender)
            ->setSubject($subject)
            ->setBody($body, $format . '/plain');
        $message->send();

        return $message->isSent();
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $variables
     * @param string $format
     * @return boolean
     */
    protected function sendTemplateMail(array $recipient, array $sender, $subject, $templateName, array $variables = array(), $format = 'text')
    {
        $emailView = $this->objectManager->create('TYPO3\CMS\Fluid\View\StandaloneView');
        $emailView->setFormat($format);

        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $templateRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['templateRootPath']);

        if (!$templateRootPath) {
            $templateRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Templates/'
            );
        }

        switch ($format)
        {
            case 'text':
                $ext = 'txt';
                break;
            case 'html':
            default:
                $ext = 'html';
                break;
        }

        $templatePathAndFilename = $templateRootPath . 'Email/' . $templateName . '.' . $ext;

        $emailView->setTemplatePathAndFilename($templatePathAndFilename);
        $emailView->assignMultiple($variables);

        return $this->sendMail($recipient, $sender, $subject, $emailView->render(), $format);
    }

    /**
     * @param string $name
     * @return array
     */
    protected function getFileArgument($name)
    {
        $fileArgument = null;

        if ($this->request->hasArgument($name) && $_FILES[$this->getPluginNamespace()]
          && $_FILES[$this->getPluginNamespace()]['name'][$name]) {
            $fileArgument = array(
                'name'      => $_FILES[$this->getPluginNamespace()]['name'][$name],
                'type'      => $_FILES[$this->getPluginNamespace()]['type'][$name],
                'tmp_name'  => $_FILES[$this->getPluginNamespace()]['tmp_name'][$name],
                'error'     => $_FILES[$this->getPluginNamespace()]['error'][$name],
                'size'      => $_FILES[$this->getPluginNamespace()]['size'][$name],
            );
        }

        return $fileArgument;
    }

    /**
     * @param array $fileData
     * @param integer $storageUid
     * @param string $folder
     * @return \TYPO3\CMS\Core\Resource\File
     */
    protected function uploadFile(array $fileData, $storageUid = 0, $folder = '')
    {
        $file = null;
        $storage = $this->getFileStorage($storageUid);

        if ($storage->isWritable() && file_exists($fileData['tmp_name'])) {
            $targetFolder = $storage->getRootLevelFolder();
            foreach (explode('/', $folder) as $folderName) {
                if ($folderName) {
                    if ($storage->hasFolderInFolder($folderName, $targetFolder)) {
                        $targetFolder = $targetFolder->getSubfolder($folderName);
                    } else {
                        $targetFolder = $targetFolder->createFolder($folderName);
                    }
                }
            }

            $file = $targetFolder->addUploadedFile($fileData, 'changeName');
            if ($file) {
                // indexes file
                $file->updateProperties(array());
            }
        }

        return $file;
    }

    /**
     * @param integer $storageUid
     * @return \TYPO3\CMS\Core\Resource\ResourceStorage
     */
    protected function getFileStorage($storageUid = 0)
    {
        if (!array_key_exists($storageUid, $this->fileStorage)) {
            $factory = $this->objectManager->get('TYPO3\CMS\Core\Resource\ResourceFactory');
            $this->fileStorage[$storageUid] = $factory->getStorageObject($storageUid);
        }

        return $this->fileStorage[$storageUid];
    }
}