<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Controller;

/**
 * @package     imia_base_ext
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class CommandController extends \TYPO3\CMS\Extbase\Mvc\Controller\CommandController
{
	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 */
	protected $configurationManager;

	/**
	 * @param \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface $configurationManager
	 */
	public function injectConfigurationManager(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface $configurationManager) {
		$this->configurationManager = $configurationManager;
		$this->settings = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
	}

    /**
     * @param integer $pageUid
     */
    protected function initFrontend($pageUid)
    {
        if (!$GLOBALS['TSFE'] || $GLOBALS['TSFE']->id != $pageUid) {
            if (!is_object($GLOBALS['TT'])) {
                $GLOBALS['TT'] = $this->objectManager->create('TYPO3\CMS\Core\TimeTracker\TimeTracker');
                $GLOBALS['TT']->start();
            }

            $GLOBALS['TSFE'] = new \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController(
                $GLOBALS['TYPO3_CONF_VARS'], $pageUid, '0', 1, '', '', '', '');

            $GLOBALS['TSFE']->initFEuser();
            $GLOBALS['TSFE']->fetch_the_id();
            $GLOBALS['TSFE']->getPageAndRootline();
            $GLOBALS['TSFE']->initTemplate();
            $GLOBALS['TSFE']->tmpl->getFileName_backPath = PATH_site;
            $GLOBALS['TSFE']->forceTemplateParsing = 1;
            $GLOBALS['TSFE']->getConfigArray();

            $this->uriBuilder = $this->objectManager->create('TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder');
            $this->uriBuilder->setRequest($this->objectManager->create('TYPO3\CMS\Extbase\Mvc\Web\Request'));
        }
    }

    /**
     * @param integer $targetPageUid
     * @param string $actionName
     * @param array $controllerArguments
     * @param string $controllerName
     * @param string $extensionName
     * @param string $pluginName
     * @param string $format
     * @param string $argumentPrefix
     * @return string
     */
    protected function uriFor($targetPageUid, $actionName = null, $controllerArguments = array(), $controllerName = null, $extensionName = null, $pluginName = null, $format = '', $argumentPrefix = null)
    {
        if ($actionName !== null) {
            $controllerArguments['action'] = $actionName;
        }
        if ($controllerName !== null) {
            $controllerArguments['controller'] = $controllerName;
        } else {
            $controllerArguments['controller'] = $this->request->getControllerName();
        }
        if ($extensionName === null) {
            $extensionName = $this->request->getControllerExtensionName();
        }
        if ($pluginName === null && TYPO3_MODE === 'FE') {
            $pluginName = \Tx_Extbase_Utility_Extension::getPluginNameByAction($extensionName, $controllerArguments['controller'], $controllerArguments['action']);
        }
        if ($pluginName === null) {
            $pluginName = $this->request->getPluginName();
        }
        if ($targetPageUid === null && TYPO3_MODE === 'FE') {
            $targetPageUid = \Tx_Extbase_Utility_Extension::getTargetPidByPlugin($extensionName, $pluginName);
        }
        if ($format !== '') {
            $controllerArguments['format'] = $format;
        }
        if ($argumentPrefix !== null) {
            $prefixedControllerArguments = array($argumentPrefix => $controllerArguments);
        } else {
            $pluginNamespace = $this->getPluginNamespace($pluginName, $extensionName);
            $prefixedControllerArguments = array($pluginNamespace => $controllerArguments);
        }

        return $prefixedControllerArguments;
    }

    /**
     * @param integer $targetPageUid
     * @param string $actionName
     * @param array $controllerArguments
     * @param string $controllerName
     * @param string $extensionName
     * @param string $pluginName
     * @param boolean $noCache
     * @param boolean $useCacheHash
     * @param boolean $createAbsoluteUri
     * @param string $format
     * @param string $argumentPrefix
     * @return string
     */
    protected function buildFrontendUri($targetPageUid, $actionName = null, $controllerArguments = array(),
        $controllerName = null, $extensionName = null, $pluginName = null, $noCache = false, $useCacheHash = true,
        $createAbsoluteUri = true, $format = '', $argumentPrefix = null)
    {
        $this->initFrontend($targetPageUid);

        $this->uriBuilder->reset();
        $this->uriBuilder->setNoCache($noCache)
                ->setUseCacheHash($useCacheHash)
                ->setCreateAbsoluteUri($createAbsoluteUri)
                ->setArguments(
            $this->uriFor($targetPageUid, $actionName, $controllerArguments, $controllerName, $extensionName, $pluginName, $format, $argumentPrefix)
        );
        $uri = $this->uriBuilder->buildFrontendUri();

         // CLI Bugfix
        $uri = str_replace(PATH_site, '/', $uri);

        return $uri;
    }

    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    protected function translate($key, $arguments = array())
    {
        return (string)\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate($key, $this->extensionName, $arguments);
    }

    /**
     * Load arguments for a different plugin than the current
     *
     * @param string $pluginName
     */
    protected function loadArguments($pluginName = null)
    {
        if ($pluginName) {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::_GPmerged($this->getPluginNamespace($pluginName));
        } else {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::array_merge_recursive_overrule($_GET, $_POST);
        }

        $this->request->setArguments($arguments);
    }

    /**
     *
     * @param string $pluginName
     * @param string $extensionName
     * @return string
     */
    protected function getPluginNamespace($pluginName = null, $extensionName = null)
    {
        $extensionService = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Service\\ExtensionService');
        $pluginNamespace = $extensionService->getPluginNamespace(
                $extensionName ?: $this->extensionName,
                $pluginName ?: $this->request->getPluginName()
        );

        return $pluginNamespace;
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $body
     * @param string $format
     * @return boolean
     */
    protected function sendMail(array $recipient, array $sender, $subject, $body, $format = 'text')
    {
        $message = $this->objectManager->get('TYPO3\CMS\Core\Mail\MailMessage');

        $message->setTo($recipient)
            ->setFrom($sender)
            ->setSubject($subject)
            ->setBody($body, $format . '/plain');
        $message->send();

        return $message->isSent();
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $variables
     * @param string $format
     * @return boolean
     */
    protected function sendTemplateMail(array $recipient, array $sender, $subject, $templateName, array $variables = array(), $format = 'text')
    {
        $emailView = $this->objectManager->create('TYPO3\CMS\Fluid\View\StandaloneView');
        $emailView->setFormat($format);

        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $templateRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['templateRootPath']);

        if (!$templateRootPath) {
            $templateRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Templates/'
            );
        }

        switch ($format)
        {
            case 'text':
                $ext = 'txt';
                break;
            case 'html':
            default:
                $ext = 'html';
                break;
        }

        $templatePathAndFilename = $templateRootPath . 'Email/' . $templateName . '.' . $ext;

        $emailView->setTemplatePathAndFilename($templatePathAndFilename);
        $emailView->assignMultiple($variables);

        return $this->sendMail($recipient, $sender, $subject, $emailView->render(), $format);
    }
}