<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Domain\Repository;

/**
 * @package     imia_base_ext
 * @subpackage  Domain\Repository
 * @author      David Frerich <d.frerich@imia.de>
 */
class Repository extends \TYPO3\CMS\Extbase\Persistence\Repository
{
    /**
     * @var array
     */
    protected $defaultOrderings = array(
        'sorting' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
    );

    /**
     * @var boolean
     */
    protected $respectEnableFields = true;

    /**
     * @var boolean
     */
    protected $respectStoragePage = true;

    /**
     * @var boolean
     */
    protected $respectSysLanguage = true;

    /**
     * @param array $uids
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByUids(array $uids, $limit = null, $offset = null, $sorting = null)
    {
        $query = $this->createQuery($limit, $offset, $sorting);
        $query->matching(
            $query->in('uid', $uids)
        );

        return $this->executeQuery($query);
    }

    /**
     * @return integer
     */
    public function findLastModified()
    {
        $query = $this->createQuery(1, 0, array('tstamp', 'desc'));
        $result = $this->executeArrayQuery($query, array('tstamp'));

        $lastModified = 0;
        foreach ($result as $row) {
            $lastModified = $row['tstamp'];
            break;
        }

        return $lastModified;
    }

    /**
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
     */
    public function createQuery($limit = null, $offset = null, $sorting = null)
    {
        $query = parent::createQuery();

        if (!$this->respectEnableFields) {
            $query->getQuerySettings()->setRespectEnableFields(false);
        }
        if (!$this->respectStoragePage) {
            $query->getQuerySettings()->setRespectStoragePage(false);
        }
        if (!$this->respectSysLanguage) {
            $query->getQuerySettings()->setRespectSysLanguage(false);
        }

		if ($limit && $limit > 0) {
			$query->setLimit($limit);
        }
		if ($offset && $offset > 0) {
			$query->setOffset($offset);
        }
        if ($sorting && is_array($sorting) && $sorting[0]) {
            $query->setOrderings(array(
                $sorting[0]	=> strtolower($sorting[1]) == 'asc' ?
                    \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING :
                    \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING,
            ));
        }

        return $query;
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
     * @return array
     */
    protected function executeUidArrayQuery($query)
    {
        $uids = array();
        $result = $this->executeArrayQuery($query, array('uid'));
        foreach ($result as $row) {
            $uids[] = $row['uid'];
        }

        return $uids;
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
     * @param array $fields
     * @return array
     */
    protected function executeArrayQuery($query, array $fields = array('uid'))
    {
        $parameters = array();
        $storageBackend = $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\Generic\Storage\Typo3DbBackend');
        $statementParts = $storageBackend->parseQuery($query, $parameters);
        $statementParts['fields']['pages'] = 'pages.' . implode(',pages.', $fields);

        $statementSql = $storageBackend->buildQuery($statementParts, $parameters);

        $query = $this->createQuery();
        $query->getQuerySettings()->setReturnRawQueryResult(true);
        $query->statement($statementSql, $parameters);

        return $query->execute();
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\Query $query
     * @param array $parameters
     * @return string
     */
    protected function getQuerySql($query, &$parameters = null)
    {
        if (!$parameters) {
            $parameters = array();
        }

        $storageBackend = $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\Generic\Storage\Typo3DbBackend');
        $statementParts = $storageBackend->parseQuery($query, $parameters);

        return $storageBackend->buildQuery($statementParts, $parameters);
    }
}